@@ -26,7 +26,7 @@ __version__ = "$Id$"
 
 import xml.etree.ElementTree as ET # Python 2.5
 
-from   layman.utils             import path, ensure_unicode
+from   layman.utils             import path
 from   layman.overlays.source   import OverlaySource, require_supported
 
 #===============================================================================
@@ -41,17 +41,11 @@ class CvsOverlay(OverlaySource):
     type = 'cvs'
     type_key = 'cvs'
 
-    def __init__(self, parent, xml, config, _location, ignore = 0, quiet = False):
+    def __init__(self, parent, config, _location, ignore = 0):
 
-        super(CvsOverlay, self).__init__(parent, xml, config, _location, ignore, quiet)
+        super(CvsOverlay, self).__init__(parent, config, _location, ignore)
+        self.subpath = None
 
-        _subpath = xml.find('subpath')
-        if _subpath != None:
-            self.subpath = ensure_unicode(_subpath.text.strip())
-        elif 'subpath' in xml.attrib:
-            self.subpath = ensure_unicode(xml.attrib['subpath'])
-        else:
-            self.subpath = ''
 
     def __eq__(self, other):
         res = super(CvsOverlay, self).__eq__(other) \
@@ -69,37 +63,55 @@ class CvsOverlay(OverlaySource):
             repo_elem.append(_subpath)
             del _subpath
 
-    def add(self, base, quiet = False):
+    def add(self, base):
         '''Add overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
+
+        cfg_opts = self.config["cvs_addopts"]
+        target = path([base, self.parent.name])
 
         # cvs [-q] co -d SOURCE SCOPE
         args = []
-        if quiet:
+        if self.config['quiet']:
             args.append('-q')
         args.append('co')
         args.append('-d')
+        if len(cfg_opts):
+            args.append(cfg_opts)
         args.append(self.parent.name)
         args.append(self.subpath)
 
-        return self.run_command(*args, cwd=base, env=dict(CVSROOT=self.src))
+        return self.postsync(
+            self.run_command(self.command(), args, cwd=base,
+                env=dict(CVSROOT=self.src), cmd=self.type),
+            cwd=target)
 
-    def sync(self, base, quiet = False):
+    def sync(self, base):
         '''Sync overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
+
+        cfg_opts = self.config["cvs_syncopts"]
+        target = path([base, self.parent.name])
 
         # cvs [-q] update -d
         args = []
-        if quiet:
+        if self.config['quiet']:
             args.append('-q')
         args.append('update')
         args.append('-d')
-        return self.run_command(*args, cwd=path([base, self.parent.name]))
+        if len(cfg_opts):
+            args.append(cfg_opts)
+        return self.postsync(
+            self.run_command(self.command(), args, cwd=target, cmd=self.type),
+            cwd=target)
 
     def supported(self):
         '''Overlay type supported?'''
 
-        return require_supported([(self.command(),  'cvs',
-                                         'dev-vcs/cvs'),])
+        return require_supported(
+            [(self.command(),  'cvs', 'dev-vcs/cvs'),],
+            self.output.warn)
