@@ -40,30 +40,59 @@ class MercurialOverlay(OverlaySource):
     type = 'Mercurial'
     type_key = 'mercurial'
 
-    def __init__(self, parent, xml, config, _location, ignore = 0, quiet = False):
+    def __init__(self, parent, config,
+        _location, ignore = 0):
 
-        super(MercurialOverlay, self).__init__(parent, xml, config, _location, ignore, quiet)
+        super(MercurialOverlay, self).__init__(parent,
+            config, _location, ignore)
+        self.subpath = None
 
-    def add(self, base, quiet = False):
+    def add(self, base):
         '''Add overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
+
+        cfg_opts = self.config["mercurial_addopts"]
+        target = path([base, self.parent.name])
+
+        if self.src.endswith("/"):
+            src = self.src
+        else:
+            src = self.src + '/'
 
         # hg clone SOURCE TARGET
-        args = ['clone', self.src + '/', path([base, self.parent.name])]
-        return self.run_command(*args)
+        if len(cfg_opts):
+            args = ['clone', cfg_opts, src, target]
+        else:
+            args = ['clone', src, target]
 
-    def sync(self, base, quiet = False):
+        return self.postsync(
+            self.run_command(self.command(), args, cmd=self.type),
+            cwd=target)
+
+    def sync(self, base):
         '''Sync overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
+
+        cfg_opts = self.config["mercurial_syncopts"]
+        target = path([base, self.parent.name])
 
         # hg pull -u SOURCE
-        args = ['pull', '-u', self.src]
-        return self.run_command(*args, cwd=path([base, self.parent.name]))
+        if len(cfg_opts):
+            args = ['pull', '-u', cfg_opts, self.src]
+        else:
+            args = ['pull', '-u', self.src]
+
+        return self.postsync(
+            self.run_command(self.command(), args, cwd=target, cmd=self.type),
+            cwd=target)
 
     def supported(self):
         '''Overlay type supported?'''
 
-        return require_supported([(self.command(),  'mercurial',
-                                         'dev-vcs/mercurial'),])
+        return require_supported(
+            [(self.command(),  'mercurial', 'dev-vcs/mercurial'),],
+            self.output.warn)
