@@ -1,8 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-#################################################################################
+################################################################################
 # LAYMAN OVERLAY BASE CLASS
-#################################################################################
+################################################################################
 # File:       overlay.py
 #
 #             Base class for the different overlay types.
@@ -20,7 +20,7 @@
 #
 ''' Basic overlay class.'''
 
-__version__ = "$Id: overlay.py 273 2006-12-30 15:54:50Z wrobel $"
+__version__ = "0.2"
 
 #===============================================================================
 #
@@ -33,13 +33,13 @@ import codecs
 import locale
 import xml.etree.ElementTree as ET # Python 2.5
 
-from   layman.utils             import ensure_unicode
-
-from   layman.debug             import OUT
+from layman.utils import pad, terminal_width, get_encoding, encoder
+from layman.compatibility import encode
 
 from   layman.overlays.bzr       import BzrOverlay
 from   layman.overlays.darcs     import DarcsOverlay
 from   layman.overlays.git       import GitOverlay
+from   layman.overlays.g_common  import GCommonOverlay
 from   layman.overlays.mercurial import MercurialOverlay
 from   layman.overlays.cvs       import CvsOverlay
 from   layman.overlays.svn       import SvnOverlay
@@ -54,6 +54,7 @@ from   layman.overlays.tar       import TarOverlay
 
 OVERLAY_TYPES = dict((e.type_key, e) for e in (
     GitOverlay,
+    GCommonOverlay,
     CvsOverlay,
     SvnOverlay,
     RsyncOverlay,
@@ -67,22 +68,20 @@ QUALITY_LEVELS = 'core|stable|testing|experimental|graveyard'.split('|')
 
 WHITESPACE_REGEX = re.compile('\s+')
 
-#===============================================================================
-#
-# Class Overlay
-#
-#-------------------------------------------------------------------------------
 
 class Overlay(object):
     ''' Derive the real implementations from this.'''
 
-    def __init__(self, xml, config, ignore = 0, quiet = False):
+    def __init__(self, config, xml=None, ovl_dict=None,
+        ignore = 0):
         '''
         >>> here = os.path.dirname(os.path.realpath(__file__))
         >>> import xml.etree.ElementTree as ET # Python 2.5
-        >>> document = ET.parse(here + '/../tests/testfiles/global-overlays.xml')
+        >>> document =ET.parse(here + '/../tests/testfiles/global-overlays.xml')
         >>> overlays = document.findall('overlay') + document.findall('repo')
-        >>> a = Overlay(overlays[0], dict())
+        >>> from layman.output import Message
+        >>> output = Message()
+        >>> a = Overlay({'output': output}, overlays[0])
         >>> a.name
         u'wrobel'
         >>> a.is_official()
@@ -95,11 +94,23 @@ class Overlay(object):
         u'Test'
         >>> a.priority
         10
-        >>> b = Overlay(overlays[1], dict())
+        >>> b = Overlay({'output': output}, overlays[1])
         >>> b.is_official()
         False
         '''
+        self.config = config
+        self.output = config['output']
+        self._encoding_ = get_encoding(self.output)
 
+        if xml is not None:
+            self.from_xml(xml, ignore)
+        elif ovl_dict is not None:
+            self.from_dict(ovl_dict, ignore)
+
+
+    def from_xml(self, xml, ignore):
+        """Process an xml overlay definition
+        """
         def strip_text(node):
             res = node.text
             if res is None:
@@ -108,36 +119,48 @@ class Overlay(object):
 
         _name = xml.find('name')
         if _name != None:
-            self.name = ensure_unicode(strip_text(_name))
+            self.name = encode(strip_text(_name))
         elif 'name' in xml.attrib:
-            self.name = ensure_unicode(xml.attrib['name'])
+            self.name = encode(xml.attrib['name'])
         else:
-            raise Exception('Overlay is missing a "name" entry!')
+            raise Exception('Overlay from_xml(), "' + self.name + \
+                'is missing a "name" entry!')
 
         _sources = xml.findall('source')
-        if _sources:
+        # new xml format
+        if _sources != []:
             _sources = [e for e in _sources if 'type' in e.attrib]
+        #old xml format
         elif ('src' in xml.attrib) and ('type' in xml.attrib):
             s = ET.Element('source', type=xml.attrib['type'])
             s.text = xml.attrib['src']
             _sources = [s]
             del s
 
-        if not _sources:
-            raise Exception('Overlay "' + self.name + '" is missing a "source" entry!')
-
-
         def create_overlay_source(source_elem):
             _type = source_elem.attrib['type']
             try:
                 _class = OVERLAY_TYPES[_type]
             except KeyError:
-                raise Exception('Unknown overlay type "%s"!' % _type)
-            _location = ensure_unicode(strip_text(source_elem))
-            return _class(self, xml, config, _location, ignore, quiet)
+                raise Exception('Overlay from_xml(), "' + self.name + \
+                    'Unknown overlay type "%s"!' % _type)
+            _location = encode(strip_text(source_elem))
+            return _class(parent=self, config=self.config,
+                _location=_location, ignore=ignore)
+
+        if not len(_sources):
+            raise Exception('Overlay from_xml(), "' + self.name + \
+                '" is missing a "source" entry!')
 
         self.sources = [create_overlay_source(e) for e in _sources]
 
+        _subpath = xml.find('subpath')
+        if _subpath != None:
+            self.subpath = encode(_subpath.text.strip())
+        elif 'subpath' in xml.attrib:
+            self.subpath = encode(xml.attrib['subpath'])
+        else:
+            self.subpath = ''
 
         _owner = xml.find('owner')
         if _owner == None:
@@ -145,49 +168,48 @@ class Overlay(object):
         else:
             _email = _owner.find('email')
         if _owner != None and _email != None:
-            self.owner_email = ensure_unicode(strip_text(_email))
+            self.owner_email = encode(strip_text(_email))
             _name = _owner.find('name')
             if _name != None:
-                self.owner_name = ensure_unicode(strip_text(_name))
+                self.owner_name = encode(strip_text(_name))
             else:
                 self.owner_name = None
         elif 'contact' in xml.attrib:
-            self.owner_email = ensure_unicode(xml.attrib['contact'])
+            self.owner_email = encode(xml.attrib['contact'])
             self.owner_name = None
         else:
             self.owner_email = ''
             self.owner_name = None
             if not ignore:
-                raise Exception('Overlay "' + self.name + '" is missing a '
-                                '"owner.email" entry!')
+                raise Exception('Overlay  from_xml(), "' + self.name + \
+                    '" is missing an "owner.email" entry!')
             elif ignore == 1:
-                OUT.warn('Overlay "' + self.name + '" is missing a '
+                self.output.warn('Overlay "' + self.name + '" is missing a '
                          '"owner.email" entry!', 4)
 
-
         _desc = xml.find('description')
         if _desc != None:
             d = WHITESPACE_REGEX.sub(' ', strip_text(_desc))
-            self.description = ensure_unicode(d)
+            self.description = encode(d)
             del d
         else:
             self.description = ''
             if not ignore:
-                raise Exception('Overlay "' + self.name + '" is missing a '
-                        '"description" entry!')
+                raise Exception('Overlay  from_xml(), "' + self.name + \
+                    '" is missing a description" entry!')
             elif ignore == 1:
-                OUT.warn('Overlay "' + self.name + '" is missing a '
+                self.output.warn('Overlay "' + self.name + '" is missing a '
                          '"description" entry!', 4)
 
         if 'status' in xml.attrib:
-            self.status = ensure_unicode(xml.attrib['status'])
+            self.status = encode(xml.attrib['status'])
         else:
             self.status = None
 
         self.quality = u'experimental'
         if 'quality' in xml.attrib:
             if xml.attrib['quality'] in set(QUALITY_LEVELS):
-                self.quality = ensure_unicode(xml.attrib['quality'])
+                self.quality = encode(xml.attrib['quality'])
 
         if 'priority' in xml.attrib:
             self.priority = int(xml.attrib['priority'])
@@ -197,13 +219,116 @@ class Overlay(object):
         h = xml.find('homepage')
         l = xml.find('link')
         if h != None:
-            self.homepage = ensure_unicode(strip_text(h))
+            self.homepage = encode(strip_text(h))
         elif l != None:
-            self.homepage = ensure_unicode(strip_text(l))
+            self.homepage = encode(strip_text(l))
         else:
             self.homepage = None
 
-        self.feeds = [ensure_unicode(strip_text(e)) for e in xml.findall('feed')]
+        self.feeds = [encode(strip_text(e)) \
+            for e in xml.findall('feed')]
+
+        _irc = xml.find('irc')
+        if _irc != None:
+            self.irc = encode(strip_text(_irc))
+        else:
+            self.irc = None
+
+
+    def from_dict(self, overlay, ignore):
+        """Process an xml overlay definition
+        """
+        self.output.debug("Overlay from_dict(); overlay" + str(overlay))
+        _name = overlay['name']
+        if _name != None:
+            self.name = encode(_name)
+        else:
+            raise Exception('Overlay from_dict(), "' + self.name +
+                'is missing a "name" entry!')
+
+        _sources = overlay['sources']
+
+        if _sources == None:
+            raise Exception('Overlay from_dict(), "' + self.name +
+                '" is missing a "source" entry!')
+
+        def create_dict_overlay_source(source_):
+            _src, _type, _sub = source_
+            try:
+                _class = OVERLAY_TYPES[_type]
+            except KeyError:
+                raise Exception('Overlay from_dict(), "' + self.name +
+                    'Unknown overlay type "%s"!' % _type)
+            _location = encode(_src)
+            return _class(parent=self, config=self.config,
+                _location=_location, ignore=ignore)
+
+        self.sources = [create_dict_overlay_source(e) for e in _sources]
+
+        _owner = overlay['owner_name']
+        if _owner == None:
+            self.owner_name = None
+            _email = None
+        else:
+            self.owner_name = encode(_owner)
+            _email = overlay['owner_email']
+        if _email != None:
+            self.owner_email = encode(_email)
+        else:
+            self.owner_email = None
+            if not ignore:
+                raise Exception('Overlay from_dict(), "' + self.name +
+                    '" is missing an "owner.email" entry!')
+            elif ignore == 1:
+                self.output.warn('Overlay from_dict(), "' + self.name +
+                    '" is missing an "owner.email" entry!', 4)
+
+        _desc = overlay['description']
+        if _desc != None:
+            d = WHITESPACE_REGEX.sub(' ', _desc)
+            self.description = encode(d)
+            del d
+        else:
+            self.description = ''
+            if not ignore:
+                raise Exception('Overlay from_dict(), "' + self.name +
+                    '" is missing a "description" entry!')
+            elif ignore == 1:
+                self.output.warn('Overlay from_dict(), "' + self.name +
+                    '" is missing a "description" entry!', 4)
+
+        if overlay['status']:
+            self.status = encode(overlay['status'])
+        else:
+            self.status = None
+
+        self.quality = u'experimental'
+        if len(overlay['quality']):
+            if overlay['quality'] in set(QUALITY_LEVELS):
+                self.quality = encode(overlay['quality'])
+
+        if overlay['priority']:
+            self.priority = int(overlay['priority'])
+        else:
+            self.priority = 50
+
+        h = overlay['homepage']
+        if h != None:
+            self.homepage = encode(h)
+        else:
+            self.homepage = None
+
+        self.feeds = [encode(e) \
+            for e in overlay['feeds']]
+
+        _irc = overlay['irc']
+        if _irc != None:
+            self.irc = encode(_irc)
+        else:
+            self.irc = None
+
+        #xml = self.to_xml()
+        # end of from_dict
 
 
     def __eq__(self, other):
@@ -218,17 +343,18 @@ class Overlay(object):
                 return False
         return True
 
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
+
     def set_priority(self, priority):
         '''Set the priority of this overlay.'''
-
         self.priority = int(priority)
 
+
     def to_xml(self):
         '''Convert to xml.'''
-
         repo = ET.Element('repo')
         if self.status != None:
             repo.attrib['status'] = self.status
@@ -244,6 +370,10 @@ class Overlay(object):
             homepage = ET.Element('homepage')
             homepage.text = self.homepage
             repo.append(homepage)
+        if self.irc != None:
+            irc = ET.Element('irc')
+            irc.text = self.irc
+            repo.append(irc)
         owner = ET.Element('owner')
         repo.append(owner)
         owner_email = ET.Element('email')
@@ -269,45 +399,46 @@ class Overlay(object):
             del feed
         return repo
 
-    def add(self, base, quiet = False):
+
+    def add(self, base):
         res = 1
+        first_s = True
         for s in self.sources:
+            if not first_s:
+                self.output.info("\nTrying next source of listed sources...", 4)
             try:
-                res = s.add(base, quiet)
+                res = s.add(base)
                 if res == 0:
                     # Worked, throw other sources away
                     self.sources = [s]
                     break
             except Exception, error:
-                OUT.warn(str(error), 4)
+                self.output.warn(str(error), 4)
+            first_s = False
         return res
 
-    def sync(self, base, quiet = False):
+
+    def sync(self, base):
+        self.output.debug("overlay.sync(); name = %s" % self.name, 4)
         assert len(self.sources) == 1
-        return self.sources[0].sync(base, quiet)
+        return self.sources[0].sync(base)
+
 
     def delete(self, base):
         assert len(self.sources) == 1
         return self.sources[0].delete(base)
 
-    def _get_encoding(self):
-        if hasattr(sys.stdout, 'encoding') \
-                and sys.stdout.encoding != None:
-            return sys.stdout.encoding
-        else:
-            return locale.getpreferredencoding()
 
-    def _encode(self, unicode_text):
-        return codecs.encode(unicode_text, self._get_encoding(), 'replace')
-
-    def __str__(self):
+    def get_infostr(self):
         '''
         >>> here = os.path.dirname(os.path.realpath(__file__))
         >>> import xml.etree.ElementTree as ET # Python 2.5
-        >>> document = ET.parse(here + '/../tests/testfiles/global-overlays.xml')
+        >>> document =ET.parse(here + '/../tests/testfiles/global-overlays.xml')
         >>> overlays = document.findall('overlay') + document.findall('repo')
-        >>> a = Overlay(overlays[0], dict())
-        >>> print str(a)
+        >>> from layman.output import Message
+        >>> output = Message()
+        >>> a = Overlay({'output': output}, overlays[0])
+        >>> print a.get_infostr()
         wrobel
         ~~~~~~
         Source  : https://overlays.gentoo.org/svn/dev/wrobel
@@ -333,13 +464,15 @@ class Overlay(object):
             result += '\n'
 
         if self.owner_name != None:
-            result += u'\nContact : %s <%s>' % (self.owner_name, self.owner_email)
+            result += u'\nContact : %s <%s>' \
+                % (self.owner_name, self.owner_email)
         else:
             result += u'\nContact : ' + self.owner_email
         if len(self.sources) == 1:
             result += u'\nType    : ' + self.sources[0].type
         else:
-            result += u'\nType    : ' + '/'.join(sorted(set(e.type for e in self.sources)))
+            result += u'\nType    : ' + '/'.join(
+                sorted(set(e.type for e in self.sources)))
         result += u'; Priority: ' + str(self.priority) + u'\n'
         result += u'Quality : ' + self.quality + u'\n'
 
@@ -359,52 +492,35 @@ class Overlay(object):
             result += u'\n  '.join((u'\n' + link).split(u'\n'))
             result += u'\n'
 
-        if self.feeds:
+        if self.irc != None:
+            result += u'\nIRC : ' + self.irc + u'\n'
+
+        if len(self.feeds):
             result += u'\n%s:' % ((len(self.feeds) == 1) and "Feed" or "Feeds")
             for i in self.feeds:
                 result += u'\n  %s' % i
             result += u'\n'
 
-        return self._encode(result)
+        return encoder(result, self._encoding_)
+
 
     def short_list(self, width = 0):
         '''
         >>> here = os.path.dirname(os.path.realpath(__file__))
         >>> import xml.etree.ElementTree as ET # Python 2.5
-        >>> document = ET.parse(here + '/../tests/testfiles/global-overlays.xml')
+        >>> document =ET.parse(here + '/../tests/testfiles/global-overlays.xml')
         >>> overlays = document.findall('repo') + document.findall('overlay')
-        >>> a = Overlay(overlays[0], dict())
+        >>> from layman.output import Message
+        >>> output = Message()
+        >>> a = Overlay({'output': output}, overlays[0])
         >>> print a.short_list(80)
         wrobel                    [Subversion] (https://o.g.o/svn/dev/wrobel         )
         '''
-
-        def pad(string, length):
-            '''Pad a string with spaces.'''
-            if len(string) <= length:
-                return string + ' ' * (length - len(string))
-            else:
-                return string[:length - 3] + '...'
-
-        def terminal_width():
-            '''Determine width of terminal window.'''
-            try:
-                width = int(os.environ['COLUMNS'])
-                if width > 0:
-                    return width
-            except:
-                pass
-            try:
-                import struct, fcntl, termios
-                query = struct.pack('HHHH', 0, 0, 0, 0)
-                response = fcntl.ioctl(1, termios.TIOCGWINSZ, query)
-                width = struct.unpack('HHHH', response)[1]
-                if width > 0:
-                    return width
-            except:
-                pass
-            return 80
-
-        name   = pad(self.name, 25)
+        if len(self.name) > 25:
+            name = self.name + "   ###\n"
+            name += pad(" ", 25)
+        else:
+            name   = pad(self.name, 25)
 
         if len(set(e.type for e in self.sources)) == 1:
             _type = self.sources[0].type
@@ -413,32 +529,40 @@ class Overlay(object):
 
         mtype  = ' [' + pad(_type, 10) + ']'
         if not width:
-            width = terminal_width()
+            width = terminal_width()-1
         srclen = width - 43
         source = ', '.join(self.source_uris())
         if len(source) > srclen:
             source = source.replace("overlays.gentoo.org", "o.g.o")
         source = ' (' + pad(source, srclen) + ')'
 
-        return self._encode(name + mtype + source)
+        return encoder(name + mtype + source, self._encoding_)
+
 
     def is_official(self):
         '''Is the overlay official?'''
-
         return self.status == 'official'
 
+
     def is_supported(self):
         return any(e.is_supported() for e in self.sources)
 
+
     def source_uris(self):
         for i in self.sources:
             yield i.src
 
-#================================================================================
+
+    def source_types(self):
+        for i in self.sources:
+            yield i.type
+
+
+#==============================================================================
 #
 # Testing
 #
-#--------------------------------------------------------------------------------
+#------------------------------------------------------------------------------
 
 if __name__ == '__main__':
     import doctest
