@@ -40,36 +40,50 @@ class RsyncOverlay(OverlaySource):
     type_key = 'rsync'
 
 
-    def __init__(self, parent, xml, config, _location, ignore = 0, quiet = False):
+    def __init__(self, parent, config, _location, ignore = 0):
 
-        super(RsyncOverlay, self).__init__(parent, xml, config, _location, ignore, quiet)
+        super(RsyncOverlay, self).__init__(parent, config,
+            _location, ignore)
+        self.subpath = None
 
-    def add(self, base, quiet = False):
+    def add(self, base):
         '''Add overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
 
         super(RsyncOverlay, self).add(base)
 
         return self.sync(base)
 
-    def sync(self, base, quiet = False):
+    def sync(self, base):
         '''Sync overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
 
         # rsync OPTIONS [-q] SOURCE TARGET
-        args = ['-rlptDvz', '--progress', '--delete', '--delete-after', '--timeout=180',
-            '--exclude=distfiles/*', '--exclude=local/*', '--exclude=packages/*']
-        if quiet:
-            args.append('-q')
-        args.append(self.src + '/')
-        args.append(path([base, self.parent.name]))
+        args = ['-rlptDvz', '--progress', '--delete', '--delete-after',
+            '--timeout=180', '--exclude=distfiles/*', '--exclude=local/*',
+            '--exclude=packages/*']
 
-        return self.run_command(*args)
+        cfg_opts = self.config["rsync_syncopts"]
+        target = path([base, self.parent.name])
+
+        if self.config['quiet']:
+            args.append('-q')
+        if len(cfg_opts):
+            args.append(cfg_opts)
+        args.append(self.src + '/')
+        args.append(target)
+
+        return self.postsync(
+            self.run_command(self.command(), args, cmd=self.type),
+            cwd=target)
 
     def supported(self):
         '''Overlay type supported?'''
 
-        return require_supported([(self.command(),  'rsync',
-                                         'net-misc/rsync'),])
+        return require_supported(
+            [(self.command(),  'rsync', 'net-misc/rsync'),],
+            self.output.warn)
