@@ -30,7 +30,12 @@ __version__ = '$Id: utils.py 236 2006-09-05 20:39:37Z wrobel $'
 #-------------------------------------------------------------------------------
 
 import types, re, os
-from layman.debug import OUT
+import sys
+import locale
+import codecs
+
+from layman.output import Message
+
 
 #===============================================================================
 #
@@ -38,11 +43,66 @@ from layman.debug import OUT
 #
 #-------------------------------------------------------------------------------
 
-def ensure_unicode(obj, encoding='utf-8'):
-    if isinstance(obj, basestring):
-        if not isinstance(obj, unicode):
-            obj = unicode(obj, encoding)
-    return obj
+def encoder(text, _encoding_):
+    return codecs.encode(text, _encoding_, 'replace')
+
+
+def decode_selection(selection):
+    '''utility function to decode a list of strings
+    accoring to the filesystem encoding
+    '''
+    # fix None passed in, return an empty list
+    selection = selection or []
+    enc = sys.getfilesystemencoding()
+    if enc is not None:
+        return [encoder(i, enc) for i in selection]
+    return selection
+
+
+def get_encoding(output):
+    if hasattr(output, 'encoding') \
+            and output.encoding != None:
+        return output.encoding
+    else:
+        encoding = locale.getpreferredencoding()
+        # Make sure that python knows the encoding. Bug 350156
+        try:
+            # We don't care about what is returned, we just want to
+            # verify that we can find a codec.
+            codecs.lookup(encoding)
+        except LookupError:
+            # Python does not know the encoding, so use utf-8.
+            encoding = 'utf_8'
+        return encoding
+
+
+def pad(string, length):
+    '''Pad a string with spaces.'''
+    if len(string) <= length:
+        return string + ' ' * (length - len(string))
+    else:
+        return string[:length - 3] + '...'
+
+
+def terminal_width():
+    '''Determine width of terminal window.'''
+    try:
+        width = int(os.environ['COLUMNS'])
+        if width > 0:
+            return width
+    except:
+        pass
+    try:
+        import struct, fcntl, termios
+        query = struct.pack('HHHH', 0, 0, 0, 0)
+        response = fcntl.ioctl(1, termios.TIOCGWINSZ, query)
+        width = struct.unpack('HHHH', response)[1]
+        if width > 0:
+            return width
+    except:
+        pass
+    return 80
+
 
 # From <http://effbot.org/zone/element-lib.htm>
 # BEGIN
@@ -103,20 +163,49 @@ def path(path_elements):
 
     return pathname
 
-def delete_empty_directory(mdir):
+def delete_empty_directory(mdir, output=None):
+    # test for a usable output parameter,
+    # and make it usable if not
+    if output is None:
+        output = Message()
     if os.path.exists(mdir) and not os.listdir(mdir):
         # Check for sufficient privileges
         if os.access(mdir, os.W_OK):
-            OUT.info('Deleting _empty_ directory "%s"' % mdir, 2)
+            output.info('Deleting _empty_ directory "%s"' % mdir, 2)
             try:
                 os.rmdir(mdir)
             except OSError, error:
-                OUT.warn(str(error))
+                output.warn(str(error))
         else:
-            OUT.warn('Insufficient permissions to delete _empty_ folder "%s".' % mdir)
+            output.warn('Insufficient permissions to delete _empty_ folder "%s".' % mdir)
             import getpass
             if getpass.getuser() != 'root':
-                OUT.warn('Hint: You are not root.')
+                output.warn('Hint: You are not root.')
+
+
+def create_overlay_dict(**kwargs):
+    """Creates a complete empty reository definition.
+    Then fills it with values passed in
+    """
+    result = {
+        'name': '',
+        'owner_name': '',
+        'owner_email': '',
+        'homepage': '',
+        'irc': '',
+        'description': '',
+        'feeds': [],
+        'sources': [('','','')],
+        'priority': 50,
+        'quality': u'experimental',
+        'status': '',
+        'official': False,
+        'supported': False,
+        }
+    for key in kwargs:
+        result[key] = kwargs[key]
+    return result
+
 
 #===============================================================================
 #
@@ -125,5 +214,5 @@ def delete_empty_directory(mdir):
 #-------------------------------------------------------------------------------
 
 if __name__ == '__main__':
-    import doctest, sys
+    import doctest
     doctest.testmod(sys.modules[__name__])
